#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#define _LIB
#define FREEIMAGE_LIB
#include "..\FreeImage\Source\FreeImage.h"

// 2004-11-29:
// - latest freeimage (3.5.2)
// - use supplied format as extension, png is default
// - getfiffromfilename instead of format - so can give jpeg or jpg, etc
// - gif is uncompressed - this is pending freeimage support
// - fixed to show error for bad output format
// 2006-11-04
// - latest freeimage (3.9.2) - real gif support
// - quantize to save gifs
// - sort list of supported formats

// comparator for sorting list of supported formats
int compare(const void *arg1, const void *arg2) {
   return _stricmp(FreeImage_GetFIFExtensionList(*(FREE_IMAGE_FORMAT*)arg1), FreeImage_GetFIFExtensionList(*(FREE_IMAGE_FORMAT*)arg2));
}

void ShowUsage() {
	printf("Usage:\tToPNG \"FileToConvert\" [output format]\n\nExample:ToPNG kitty.jpg gif\nMany:\tfor %%a in (*.bmp) do TOPNG %%a jpg\n\n");

	int count = FreeImage_GetFIFCount();

	// sort list of supported formats
	FREE_IMAGE_FORMAT* fifs = new FREE_IMAGE_FORMAT[count];
	for(int x=0; x<count; x++)
		fifs[x] = (FREE_IMAGE_FORMAT)x;
	qsort(fifs, count, sizeof(FREE_IMAGE_FORMAT), compare);

	// display sorted list of formats
	printf("%20s\tOutput\tDescription\n", "Format");
	printf("%20s\t------\t-----------\n", "------");
	for(int x=0; x<count; x++) {
		if(x==0 || _stricmp(FreeImage_GetFIFExtensionList(fifs[x]),FreeImage_GetFIFExtensionList(fifs[x-1]))!=0)
			printf("%20s\t%s\t%s\n", FreeImage_GetFIFExtensionList(fifs[x]),
				(FreeImage_FIFSupportsWriting(fifs[x])?"x":"-"),
				FreeImage_GetFIFDescription(fifs[x]) );
	}

	delete[] fifs;
}

long fsize(const char* name) {
	FILE* infile;
	long length=0;
	infile = fopen(name, "rb");
	if(infile!=NULL) {
		if(fseek(infile, 0, SEEK_END)==0)
			length=ftell(infile);
		fclose(infile);
	}
	return length;
}

void FreeImageErrorHandler(FREE_IMAGE_FORMAT fif, const char *message) {
	printf("\n*** %s Format\n%s ***\n",
		FreeImage_GetFormatFromFIF(fif), message);
}

int main(char argc, char *argv[]) {
	FreeImage_Initialise(FALSE);

	FreeImage_SetOutputMessage(FreeImageErrorHandler);

	if(argc < 2) {
		ShowUsage();
		return 1;
	}

	FREE_IMAGE_FORMAT fifTo=FIF_UNKNOWN;
	if(argc > 2) {
		fifTo = FreeImage_GetFIFFromFilename(argv[2]);
		if(fifTo==FIF_UNKNOWN || !FreeImage_FIFSupportsWriting(fifTo)) {
			printf("%s : unsupported output format\n", argv[2]);
			return 1;
		}
	} else 
		fifTo = FreeImage_GetFIFFromFormat("PNG");

	FREE_IMAGE_FORMAT fif = FreeImage_GetFIFFromFilename(argv[1]);
	if(fif==FIF_UNKNOWN) {
		printf("Unknown file format: %s\n", argv[1]);
		return 2;
	}

	FIBITMAP* fib = FreeImage_Load(fif, argv[1]);
	if(!fib) {
		printf("Error loading file: %s\n", argv[1]);
		return 3;
	}

	long filesizebefore, filesizeafter;
	filesizebefore=fsize(argv[1]);

	char szFile[2048];
	char* p = strrchr(argv[1], '.');
	if(*p) {
		memcpy(szFile, argv[1], p-argv[1]);
		szFile[p-argv[1]]=0;
	} else strcpy(szFile, argv[1]);
	strcat(szFile, ".");

	if(FreeImage_GetBPP(fib)!=24 && (FreeImage_GetFormatFromFIF(fifTo)=="JPEG" || FreeImage_GetFormatFromFIF(fifTo)=="PNG")) {
		FIBITMAP* fib2 = FreeImage_ConvertTo24Bits(fib);
		FreeImage_Unload(fib);
		fib=fib2;
	}
	else if(FreeImage_GetBPP(fib)>8 && FreeImage_GetFormatFromFIF(fifTo)=="GIF") {
		printf("Reducing to 256 colors for GIF\n");
		FIBITMAP* fib2 = FreeImage_ColorQuantize(fib, FIQ_WUQUANT);
		FreeImage_Unload(fib);
		fib=fib2;
	}

	if(argc==2)
		strcat(szFile, "png");
	else
		strcat(szFile, argv[2]);

	printf("Converting %s from %s to %s\n", argv[1], FreeImage_GetFormatFromFIF(fif), FreeImage_GetFormatFromFIF(fifTo));

	if(!FreeImage_FIFSupportsExportBPP(fifTo, FreeImage_GetBPP(fib))) {
		printf("Output format does not support the bit depth of this image.\n", argv[1]);
		return 4;
	}

	FreeImage_Save(fifTo, fib, szFile);
	FreeImage_Unload(fib);
	filesizeafter=fsize(szFile);
	printf("%s: %.2fk\n%s: %.2fk (%.1f%%)\n",
		argv[1], (float)filesizebefore/1024,
		szFile, (float)filesizeafter/1024,
		(float)filesizeafter/filesizebefore*100 );

	return 0;
}